@@ -11,7 +11,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: common.py,v 1.13 2004/07/27 21:30:53 atzm Exp $
+# $Id: common.py,v 1.17 2004/08/27 18:14:03 tkawa Exp $
 #
 
 import os, sys
@@ -19,7 +19,7 @@ import gtk
 
 # common defines
 APP = 'GBottler'
-VER = '0.0.7'
+VER = '0.1.0'
 
 VOTE  = "Vote"
 AGREE = "Agree"
@@ -63,6 +63,7 @@ DL_SUFFIX_PATTERNS = {
 	}
 
 WAIT_NEXT_REQUEST = 20
+WAIT_NEXT_LOCALGHOST_POLL = 30000
 
 # common functions
 def open_bottlecase(path=None):
@@ -86,11 +87,11 @@ def get_icon_button(stock, size=gtk.ICON_SIZE_MENU,
 		h = gtk.HBox(gtk.FALSE, 0)
 		h.set_border_width(0)
 		h.show()
-		h.pack_start(i, gtk.FALSE, gtk.TRUE, 0)
+		h.pack_start(i, gtk.TRUE, gtk.TRUE, 0)
 
 		l = gtk.Label(label)
 		l.show()
-		h.pack_start(l, gtk.FALSE, gtk.TRUE, 0)
+		h.pack_start(l, gtk.TRUE, gtk.TRUE, 0)
 		b.add(h)
 	else:
 		b.add(i)
@@ -171,3 +172,25 @@ def open_error_dialog(label, parent=None, stock=gtk.STOCK_DIALOG_ERROR):
 	d.destroy()
 	d = None
 	return
+
+def open_simple_entry_dialog(title, label, parent=None, stock=gtk.STOCK_DIALOG_QUESTION):
+	d = gtk.Dialog(title, parent,
+				   gtk.DIALOG_DESTROY_WITH_PARENT|gtk.DIALOG_MODAL|gtk.DIALOG_NO_SEPARATOR,
+				   (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT, gtk.STOCK_OK, gtk.RESPONSE_ACCEPT))
+	d.connect("destroy", lambda w: w.destroy)
+	h = get_icon_box(stock, label, gtk.ICON_SIZE_DIALOG)
+	d.vbox.pack_start(h)
+	e = gtk.Entry()
+	e.show()
+	d.vbox.pack_start(e)
+	d.show()
+
+	response = d.run()
+	if response == gtk.RESPONSE_ACCEPT:
+		result = e.get_text()
+	else:
+		result = ''
+	d.hide()
+	d.destroy()
+	d = None
+	return result
