@@ -13,7 +13,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: gbottler.py,v 1.115 2004/07/27 20:04:43 atzm Exp $
+# $Id: gbottler.py,v 1.126 2004/08/27 18:18:34 tkawa Exp $
 #
 
 import os
@@ -96,15 +96,15 @@ class Bottler(BottleClient):
 
 			# Forward all messages?
 			if self.app.prefs.get('accept_all'):
-				gtk.timeout_add(0, self.app.send_local_message, channel, ifghost, script, sender, charset)
+				gtk.timeout_add(WAIT_NEXT_REQUEST, self.app.send_local_message, channel, ifghost, script, sender, charset)
 
 			# Forward only installed ghosts?
 			elif self.app.prefs.get('forward_listed'):
 				if ifghost in self.app.local_ghost:
-					gtk.timeout_add(0, self.app.send_local_message, channel, ifghost, script, sender, charset)
+					gtk.timeout_add(WAIT_NEXT_REQUEST, self.app.send_local_message, channel, ifghost, script, sender, charset)
 
 		# log message
-		gtk.timeout_add(0, self.app.log_window.update, mid, channel, ifghost, script, receive_time)
+		gtk.timeout_add(WAIT_NEXT_REQUEST, self.app.log_window.update, mid, channel, ifghost, script, receive_time)
 
 		# download
 		if self.app.prefs.get('autodl'):
@@ -118,7 +118,7 @@ class Bottler(BottleClient):
 	def handle_broadcast_information(self, type, forced):
 		BottleClient.handle_broadcast_information(self, type, forced)
 		if type in [VOTE, AGREE]:
-			gtk.timeout_add(0, self.app.log_votes, self.headers["mid"], type, int(self.headers["num"]))
+			gtk.timeout_add(WAIT_NEXT_REQUEST, self.app.log_votes, self.headers["mid"], type, int(self.headers["num"]))
 
 	def handle_all_users(self):
 		self.app.show_users()
@@ -167,29 +167,26 @@ class Application:
 
 		for m in ["/%s/%s" % (self.msg["file"][1], self.msg["part"]),
 				  "/%s" % self.msg["channel"][1],
-				  "/%s" % self.msg["ghost"][1]]:
-			# part, channel and ghost are not sensitive at startup
-			self.menu_list[m].set_sensitive(gtk.FALSE)			
+				  "/%s/%s" % (self.msg["edit"][1], self.msg["send"])]:
+			# part, channel and send are not sensitive at startup
+			self.menu_list[m].set_sensitive(gtk.FALSE)
 
 		main_menubar = self.item_factory.get_widget("<main>")
 		main_menubar.show()
 
 # === Create Main Editor === #
-		self.main_text = gtk.TextView()
-		self.main_text.show()
-		self.main_text.set_size_request(500,100)
-		self.main_text.set_wrap_mode(gtk.WRAP_CHAR)
-
-		self.textmanager = TextManager(self, self.main_text, self.prefs)
+		self.textmanager = TextManager(self, self.prefs)
 		self.textmanager.set_style_script()
+		self.textmanager.set_size_request(500, 100)
+		self.textmanager.show()
 
-		self.editor = self.main_text.get_buffer()
+		self.editor = self.textmanager.get_buffer()
 		self.editor.connect('end-user-action', self.editor_modified)
 
 		sw = gtk.ScrolledWindow()
 		sw.show()
 		sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-		sw.add(self.main_text)
+		sw.add(self.textmanager)
 
 		text_frame = gtk.Frame()
 		text_frame.show()
@@ -213,10 +210,10 @@ class Application:
 		self.send_button.connect("clicked", self.send_question, None)
 		self.clear_button.connect("clicked", self.edit_clear, None)
 
-		checkButton.connect("focus-in-event", lambda x, y: self.main_text.grab_focus())
-		testButton.connect("focus-in-event", lambda x, y: self.main_text.grab_focus())
-		self.send_button.connect("focus-in-event", lambda x, y: self.main_text.grab_focus())
-		self.clear_button.connect("focus-in-event", lambda x, y: self.main_text.grab_focus())
+		checkButton.connect("focus-in-event", lambda x, y: self.textmanager.grab_focus())
+		testButton.connect("focus-in-event", lambda x, y: self.textmanager.grab_focus())
+		self.send_button.connect("focus-in-event", lambda x, y: self.textmanager.grab_focus())
+		self.clear_button.connect("focus-in-event", lambda x, y: self.textmanager.grab_focus())
 
 		self.tooltips.set_tip(checkButton, unicode(_("Check"), "utf-8"))
 		self.tooltips.set_tip(testButton, unicode(_("Test"), "utf-8"))
@@ -279,14 +276,7 @@ class Application:
 		toolframe.add(main_toolbar)
 
 # === Create Popup Menu from Main Editor === #
-		pmsg = self.get_popup_menu_messages()
-		popup_menu_items = self.get_popup_menu_items(pmsg)
-
-		self.popup_item_factory = gtk.ItemFactory(gtk.Menu, "<main>", accel_group)
-		self.popup_item_factory.create_items(popup_menu_items)
-		self.popup_menu = self.popup_item_factory.get_widget("<main>")
-
-		self.main_text.connect("button-press-event", self.popup)
+		self.textmanager.connect("populate-popup", self.popup)
 
 # === Create Progressbar === #
 		self.pbar = gtk.ProgressBar()
@@ -322,7 +312,7 @@ class Application:
 		main_vbox.pack_start(main_paned, gtk.TRUE, gtk.TRUE, 0) # main editor and progressbar
 
 		self.window.show()
-		self.main_text.grab_focus()
+		self.textmanager.grab_focus()
 
 # === Initialization of Application === #
 		[str, pos] = self.get_initial_script()
@@ -346,7 +336,7 @@ class Application:
 		self.gvwindow = GVWindow(os.path.join(open_bottlecase(), 'gviewer', 'Ghost'),
 								 os.path.join(open_bottlecase(), 'gviewer', 'ghost.txt'))
 
-		self.svgmanager = SVGManager(self, os.path.join(open_bottlecase(), 'gviewer'))
+		self.svgmanager = SVGManager(self, self.prefs, os.path.join(open_bottlecase(), 'gviewer'))
 
 		self.menu_list["/%s/%s" % (self.msg["window"][1],
 								   self.msg["logs"])].connect("activate", self.log_window.open, None)
@@ -361,21 +351,20 @@ class Application:
 								   self.msg["svgmanager"])].connect("activate", self.svgmanager.open, None)
 
 		self.execute_command = 'GetNames'
-		self.target_ghost = ""
-		self.sakuraname = ""
+		self.target_ghost = self.DEFAULT_TARGET
 
 		self.local_ghost = []
-		self.local_ghost_last_notified = 0
-
 		self.fmo_ghosts = self.gvwindow.get_fmo_ghosts()
 
-		self.send_ghost_names()
-
 		self.c_e_id = 1 # channel label eventbox handler id
 		self.g_e_id = 1
 		self.disable_channel()
 
+		self.send_ghost_names()
+		self.rebuild_ghostmenu()
+
 		self.timeout_id = None
+		self.localghost_timeout_id = gtk.timeout_add(WAIT_NEXT_LOCALGHOST_POLL, self.handle_localghost)
 
 # === Initialization from Preferences === #
 		if self.prefs.get('auto_open_log'):
@@ -408,15 +397,28 @@ class Application:
 	def phrase_open(self, widget=None, data=None):
 		PhraseWindow(self, PhraseManager()).open()
 
+	def save_phrases(self, data, widget):
+		regexp = re.compile('\r?\n')
+
+		title = regexp.sub('', open_simple_entry_dialog(unicode(_('Please input title'), 'utf-8'),
+														unicode(_('Please input title'), 'utf-8'),
+														self.window))
+		script = regexp.sub('', self.textmanager.get_script())
+
+		if title and script:
+			pm = PhraseManager()
+			pm.append("%s\t%s" % (title, script))
+			pm.save_database()
+
 	def get_initial_script(self):
 		scr = self.prefs.get('initialscript')
 		n = string.find(scr, '|')
 
-		# | founds
+		# | found
 		if n != -1:
 			n -= len(scr)
 
-			# | founds at rightmost
+			# | found at rightmost
 			if n != -1:
 				scr = scr[:n] + scr[n+1:]
 			else:
@@ -429,19 +431,22 @@ class Application:
 		[si, ei] = self.editor.get_bounds()
 		script = self.editor.get_text(si, ei)
 		self.sbar.pop(0)
-		self.sbar.push(0, str(len(script.encode('sjis'))) + unicode(_('bytes'), 'utf-8'))
+		try:
+			self.sbar.push(0, str(len(script.encode('sjis'))) + unicode(_('bytes'), 'utf-8'))
+		except:
+			self.sbar.push(0, str(len(script)) + unicode(_('bytes'), 'utf-8'))
 
 	def edit_cut(self, widget, data=None):
-		self.main_text.emit("cut-clipboard")
+		self.textmanager.emit("cut-clipboard")
 
 	def edit_copy(self, widget, data=None):
-		self.main_text.emit("copy-clipboard")
+		self.textmanager.emit("copy-clipboard")
 
 	def edit_paste(self, widget, data=None):
-		self.main_text.emit("paste-clipboard")
+		self.textmanager.emit("paste-clipboard")
 
 	def edit_delete(self, widget, data=None):
-		self.main_text.emit("delete-from-cursor", gtk.DELETE_CHARS, 1)
+		self.textmanager.emit("delete-from-cursor", gtk.DELETE_CHARS, 1)
 
 	def open_preference(self, widget=None, data=None):
 		PreferenceDialog(self, self.prefs, self.luid_info)
@@ -476,9 +481,36 @@ class Application:
 		if self.client.connected:
 			self.rebuild_ghostmenu()
 
- 	def popup(self, widget=None, event=None):
- 		if event.button == 3:
- 			self.popup_menu.popup(None, widget, None, event.button, 0)
+	def popup(self, textview, menu):
+		accel_group = gtk.AccelGroup()
+		popup_menu_items = self.get_popup_menu_items()
+		popup_item_factory = gtk.ItemFactory(gtk.Menu, "<main>", accel_group)
+		popup_item_factory.create_items(popup_menu_items)
+		popup_menu = popup_item_factory.get_widget("<main>")
+
+		orig_menu = []
+		for child in menu.get_children():
+			orig_menu.append(child)
+			menu.remove(child)
+
+		children = popup_menu.get_children()
+		children.reverse()
+		for child in children:
+			popup_menu.remove(child)
+			menu.prepend(child)
+
+		for item in orig_menu:
+			menu.get_children()[-1].get_submenu().append(item)
+
+		# check channel permission
+		send_item = popup_item_factory.get_item("/%s/%s" % (self.msg["edit"][1], self.msg["send"]))
+		if not self.client.connected:
+			send_item.set_sensitive(gtk.FALSE)
+		else:
+			if self.client.channels[self.get_channel_id()]["nopost"]:
+				send_item.set_sensitive(gtk.FALSE)
+			else:
+				send_item.set_sensitive(gtk.TRUE)
 
 	def join(self, widget=None, data=None):
 		if not self.client.initialized:
@@ -513,7 +545,7 @@ class Application:
 		self.pbar.set_fraction(0.0)
 		self.monitor_clear()
 		self.enable_channel()
-		self.timeout_id = gtk.timeout_add(100, self.handle_event)
+		self.timeout_id = gtk.timeout_add(WAIT_NEXT_REQUEST, self.handle_event)
 
 	def button_popup(self, widget, event, sender):
 		if event.button in [1, 3]:
@@ -589,12 +621,14 @@ class Application:
 			self.g_e_box.disconnect(self.g_e_id)
 
 		current = self.target_ghost
-		ghosts = [self.DEFAULT_TARGET] + self.uniq_ghosts() #self.local_ghost + self.fmo_ghosts
+		ghosts = [self.DEFAULT_TARGET] + self.uniq_ghosts()
 		name = ""
-		for n in range(len(self.client.channels)):
-			name = self.client.channels[n]["ghost"]
-			if name and name not in ghosts:
-				ghosts.append(name)
+
+		if self.client.connected:
+			for n in range(len(self.client.channels)):
+				name = self.client.channels[n]["ghost"]
+				if name and name not in ghosts:
+					ghosts.append(name)
 
 		ghost_menu = []
 		for n in range(len(ghosts)):
@@ -617,12 +651,8 @@ class Application:
 			all_ghost_items[current].set_active(gtk.TRUE)
 			all_ghost_items[current].toggled()
 		else:
-			if current != self.DEFAULT_TARGET and current != name:
-				all_ghost_items[self.sakuraname].set_active(gtk.TRUE)
-				all_ghost_items[self.sakuraname].toggled()
-			else:
-				all_ghost_items[self.DEFAULT_TARGET].set_active(gtk.TRUE)
-				all_ghost_items[self.DEFAULT_TARGET].toggled()
+			all_ghost_items[self.DEFAULT_TARGET].set_active(gtk.TRUE)
+			all_ghost_items[self.DEFAULT_TARGET].toggled()
 
 		ghost_item.set_submenu(self.g_ifact.get_widget("<main>"))
 		ghost_item.set_sensitive(gtk.TRUE)
@@ -630,7 +660,8 @@ class Application:
 		if not self.g_e_box.handler_is_connected(self.g_e_id):
 			self.g_e_id = self.g_e_box.connect("button-press-event",
 											   self.button_popup, self.g_ifact.get_widget("<main>"))
-		self.check_channel_permission()
+		if self.client.connected:
+			self.check_channel_permission()
 
 	def update_ghost(self, widget=None, data=None):
 		self.target_ghost = data
@@ -640,8 +671,15 @@ class Application:
 		channel = self.get_channel_id()
 		if self.client.channels[channel]["nopost"]:
 			self.send_button.set_sensitive(gtk.FALSE)
+			self.menu_list["/%s/%s" % (self.msg["edit"][1], self.msg["send"])].set_sensitive(gtk.FALSE)
 		else:
 			self.send_button.set_sensitive(gtk.TRUE)
+			self.menu_list["/%s/%s" % (self.msg["edit"][1], self.msg["send"])].set_sensitive(gtk.TRUE)
+
+	def handle_localghost(self):
+		self.send_ghost_names()
+		self.rebuild_ghostmenu()
+		self.localghost_timeout_id = gtk.timeout_add(WAIT_NEXT_LOCALGHOST_POLL, self.handle_localghost)
 
 	def handle_event(self):
 		if not self.client.connected:
@@ -654,12 +692,7 @@ class Application:
 			self.disable_channel()
 			return
 
-		now = time.time()
-		if now - self.local_ghost_last_notified > 30: # about 30 seconds
-			self.local_ghost_last_notified = now
-			self.send_ghost_names()
-			self.rebuild_ghostmenu()
-		self.timeout_id = gtk.timeout_add(1000, self.handle_event)
+		self.timeout_id = gtk.timeout_add(WAIT_NEXT_REQUEST, self.handle_event)
 
 	SSTP_EXECUTE = (
 		"EXECUTE SSTP/1.0\r\n"
@@ -712,18 +745,17 @@ class Application:
 			return re.split('\s*,\s*', names[0], 1)
 
 	def send_ghost_names(self):
-		ghosts =  self.get_execute('GetNames')
-		sakura, kero = self.get_execute('GetName')
-
+		ghosts = self.get_execute('GetNames')
 		if ghosts:
 			self.local_ghost = ghosts
-			self.sakuraname  = sakura
-		elif sakura:
+			return
+
+		sakura, kero = self.get_execute('GetName')
+		if sakura:
 			self.local_ghost = [sakura]
-			self.sakuraname  = sakura
-		else:
-			self.local_ghost = []
-			self.sakuraname  = self.DEFAULT_TARGET
+			return
+
+		self.local_ghost = []
 
 	def monitor_clear(self):
 		self.pbar.set_text("")
@@ -747,13 +779,12 @@ class Application:
 			self.g_e_id = 1
 
 		self.send_button.set_sensitive(gtk.FALSE)
+		self.menu_list["/%s/%s" % (self.msg["edit"][1], self.msg["send"])].set_sensitive(gtk.FALSE)
 		self.channel.set_text("")
-		self.channel_ghost.set_text("")
-		self.target_ghost = self.DEFAULT_TARGET
+
 		self.users.set_text("")
 
-		for m in ["/%s" % self.msg["ghost"][1],
-				  "/%s" % self.msg["channel"][1]]:
+		for m in ["/%s" % self.msg["channel"][1]]:
 			menu = self.menu_list[m]
 			menu.deselect()
 			menu.set_sensitive(gtk.FALSE)
@@ -808,7 +839,7 @@ class Application:
 		self.sbar.push(0, '0' + unicode(_('bytes'), 'utf-8'))
 
 	def check(self, widget=None, data=None):
-		self.main_text.grab_focus()
+		self.textmanager.grab_focus()
 		self.monitor_clear()
 		self.monitor_insert(unicode(_("Now checking script..."), "utf8"))
 		[si, ei] = self.editor.get_bounds()
@@ -949,14 +980,14 @@ class Application:
 			return 1
 		return 0
 
-	def test(self, widget, data):
-		self.main_text.grab_focus()
+	def test(self, widget=None, data=None):
+		self.textmanager.grab_focus()
 		if self.check(widget, data):
 			return
 		self.send_local_message('[' + unicode(_("Test"), "utf-8") + ']', self.target_ghost, self.get_script())
 
-	def send_question(self, widget, data):
-		self.main_text.grab_focus()
+	def send_question(self, widget=None, data=None):
+		self.textmanager.grab_focus()
 		if self.check(widget, data):
 			return
 
@@ -977,11 +1008,9 @@ class Application:
 		ghost = self.target_ghost
 		if ghost == self.DEFAULT_TARGET:
 			ghost = None
-		gtk.timeout_add(0, self.send, widget, data, channel, script, ghost)
+		gtk.timeout_add(WAIT_NEXT_REQUEST, self.send, widget, data, channel, script, ghost)
 
 	def send(self, widget, data, channel, script, ghost):
-		self.sent_log(channel, script, ghost)
-
 		self.monitor_clear()
 		self.monitor_insert(unicode(_("Broadcasting..."), "utf8"))
 		try:
@@ -993,12 +1022,14 @@ class Application:
 			error = str(e)
 
 		if error is not None:
-			self.monitor_insert(unicode(_("failed!!")) + "\n" + \
-								unicode(_("Error"), "utf8") + ": ")
-			self.monitor_insert(unicode(error))
+			open_error_dialog(unicode(_("failed!!")) + "\n" + \
+							  unicode(_("Error"), "utf8") + ": \n" + \
+							  unicode(error), self.window)
+		else:
+			gtk.timeout_add(WAIT_NEXT_REQUEST, self.sent_log, channel, script, ghost)
 
-		if self.prefs.get('auto_clear'):
-			self.edit_clear(data, widget)
+			if self.prefs.get('auto_clear'):
+				self.edit_clear(data, widget)
 
 		self.pbar.set_fraction(0.0)
 
@@ -1083,9 +1114,13 @@ class Application:
 		"Charset: %s\r\n"
 		"\r\n")
 	def send_local_message(self, channel, ifghost, script, sender=APP, charset='Shift_JIS'):
-		result = self.client.send_sstp_message(self.SSTP_SEND % (sender, channel, ifghost, ifghost, script, charset),
-											   self.host, self.port)
-		#print result
+		try:
+			result = self.client.send_sstp_message(self.SSTP_SEND % (sender, channel, ifghost, ifghost, script, charset),
+												   self.host, self.port)
+		except BottleClientError, e:
+			open_error_dialog(str(e), self.window)
+			return
+
 		if self.prefs.get('use_gviewer'):
 			if self.prefs.get('open_gviewer_when_play'):
 				self.gvwindow.open()
@@ -1099,11 +1134,11 @@ class Application:
 		self.pbar.set_fraction(0.0)
 
 	def insert(self, pre, post=""):
-		self.main_text.emit("insert-at-cursor", pre)
+		self.textmanager.emit("insert-at-cursor", pre)
 		if post:
-			self.main_text.emit("insert-at-cursor", post)
-			self.main_text.emit("move-cursor", gtk.MOVEMENT_LOGICAL_POSITIONS,
-								-1 * len(post), gtk.FALSE)
+			self.textmanager.emit("insert-at-cursor", post)
+			self.textmanager.emit("move-cursor", gtk.MOVEMENT_LOGICAL_POSITIONS,
+								  -1 * len(post), gtk.FALSE)
 
 	def insert_h(self, data, widget): self.insert("\n\\h")
 	def insert_u(self, data, widget): self.insert("\n\\u")
@@ -1150,8 +1185,45 @@ class Application:
 		gtk.main()
 		return 0
 
-	def get_popup_menu_messages(self):
-		pmsg = {
+	def set_self_messages(self):
+		self.msg = {
+			"file":   [unicode(_("File(_F)"), "utf-8"),
+					   unicode(_("File(F)"), "utf-8")],
+			"join":   unicode(_("Join"), "utf-8"),
+			"part":   unicode(_("Part"), "utf-8"),
+			"quit":   unicode(_("Quit"), "utf-8"),
+			"edit":   [unicode(_("Edit(_E)"), "utf-8"),
+					   unicode(_("Edit(E)"), "utf-8")],
+
+			"check":  unicode(_("Check"), 'utf-8'),
+			"test":   unicode(_("Test"), 'utf-8'),
+			"send":   unicode(_("Send"), 'utf-8'),
+
+			"cut":	  unicode(_("Cut"), "utf-8"),
+			"copy":   unicode(_("Copy"), "utf-8"),
+			"paste":  unicode(_("Paste"), "utf-8"),
+			"delete": unicode(_("Delete"), "utf-8"),
+			"save phrases": unicode(_("Save phrases"), 'utf-8'),
+			"clear editor": unicode(_("Clear Editor"), "utf-8"),
+			"clear editor all": unicode(_("Clear Editor All"), "utf-8"),
+			"pref":   [unicode(_("Preferences(_P)"), "utf-8"),
+					   unicode(_("Preferences(P)"), "utf-8")],
+			"channel":[unicode(_("Channel(_C)"), "utf-8"),
+					   unicode(_("Channel(C)"), "utf-8")],
+			"ghost":  [unicode(_("Ghost(_G)"), "utf-8"),
+					   unicode(_("Ghost(G)"), "utf-8")],
+			"window": [unicode(_("Window(_W)"), "utf-8"),
+					   unicode(_("Window(W)"), "utf-8")],
+			"logs":    unicode(_("Logs"), "utf-8"),
+			"phrases": unicode(_("Phrases"), "utf-8"),
+			"gviewer": unicode(_("GViewer"), "utf-8"),
+			"svgmanager": unicode(_("SVGManager"), "utf-8"),
+			"help":   [unicode(_("Help(_H)"), "utf-8"),
+					   unicode(_("Help(H)"), "utf-8")],
+			"about":  unicode(_("About"), "utf-8"),
+		}
+	
+		self.pmsg = {
      		"sessions":	    unicode(_("Sessions"), "utf-8"),
      		"timecritical": unicode(_("TimeCritical"), "utf-8"),
      		"quick":		unicode(_("Quick"), "utf-8"),
@@ -1189,132 +1261,113 @@ class Application:
      
      		"terminate":	unicode(_("Terminate"), "utf-8"),
      		}
-		return pmsg
-	
-	def get_popup_menu_items(self, pmsg):     
+
+	def get_popup_menu_items(self):     
 		popup_menu_items = (
-     		( "/sep0", None, None, 0, "<Separator>" ),
-     		( "/%s" % pmsg["sessions"], None, None, 0, "<Branch>" ),
-     		( "/%s/%s" % (pmsg["sessions"], pmsg["timecritical"]), None, self.insert_t, 0, None ),
-     		( "/%s/%s" % (pmsg["sessions"], pmsg["quick"]), None, self.insert_q, 0, None ),
-     		( "/%s/%s" % (pmsg["sessions"], pmsg["syncronized"]), None, self.insert_s, 0, None ),
+     		( "/%s" % self.pmsg["sessions"], None, None, 0, "<Branch>" ),
+     		( "/%s/%s" % (self.pmsg["sessions"], self.pmsg["timecritical"]), None, self.insert_t, 0, None ),
+     		( "/%s/%s" % (self.pmsg["sessions"], self.pmsg["quick"]), None, self.insert_q, 0, None ),
+     		( "/%s/%s" % (self.pmsg["sessions"], self.pmsg["syncronized"]), None, self.insert_s, 0, None ),
      		( "/sep1", None, None, 0, "<Separator>" ),
      
-     		( "/%s" % pmsg["sakura"], None, self.insert_h, 0, None ),
-     		( "/%s" % pmsg["countenance"], None, None, 0, "<Branch>" ),
-     		( "/%s/[0]  %s" % (pmsg["countenance"], pmsg["ordinary"]),
+     		( "/%s" % self.pmsg["sakura"], None, self.insert_h, 0, None ),
+     		( "/%s" % self.pmsg["countenance"], None, None, 0, "<Branch>" ),
+     		( "/%s/[0]  %s" % (self.pmsg["countenance"], self.pmsg["ordinary"]),
 			  "<control><alt>0",self.insert_s0,  0, None, ),
-     		( "/%s/[1]  %s" % (pmsg["countenance"], pmsg["shy"]),
+     		( "/%s/[1]  %s" % (self.pmsg["countenance"], self.pmsg["shy"]),
 			  "<control><alt>1", self.insert_s1, 0, None, ),
-     		( "/%s/[2]  %s" % (pmsg["countenance"], pmsg["surprised"]),
+     		( "/%s/[2]  %s" % (self.pmsg["countenance"], self.pmsg["surprised"]),
 			  "<control><alt>2", self.insert_s2, 0, None, ),
-     		( "/%s/[3]  %s" % (pmsg["countenance"], pmsg["angst"]),
+     		( "/%s/[3]  %s" % (self.pmsg["countenance"], self.pmsg["angst"]),
 			  "<control><alt>3", self.insert_s3, 0, None, ),
-     		( "/%s/[4]  %s" % (pmsg["countenance"], pmsg["ease off"]),
+     		( "/%s/[4]  %s" % (self.pmsg["countenance"], self.pmsg["ease off"]),
 			  "<control><alt>4", self.insert_s4, 0, None, ),
-     		( "/%s/[5]  %s" % (pmsg["countenance"], pmsg["smile"]),
+     		( "/%s/[5]  %s" % (self.pmsg["countenance"], self.pmsg["smile"]),
 			  "<control><alt>5", self.insert_s5, 0, None, ),
-     		( "/%s/[6]  %s" % (pmsg["countenance"], pmsg["closed eyes"]),
+     		( "/%s/[6]  %s" % (self.pmsg["countenance"], self.pmsg["closed eyes"]),
 			  "<control><alt>6", self.insert_s6, 0, None, ),
-     		( "/%s/[7]  %s" % (pmsg["countenance"], pmsg["angry"]),
+     		( "/%s/[7]  %s" % (self.pmsg["countenance"], self.pmsg["angry"]),
 			  "<control><alt>7", self.insert_s7, 0, None, ),
-     		( "/%s/[8]  %s" % (pmsg["countenance"], pmsg["scorned"]),
+     		( "/%s/[8]  %s" % (self.pmsg["countenance"], self.pmsg["scorned"]),
 			  "<control><alt>8", self.insert_s8, 0, None, ),
-     		( "/%s/[20] %s" % (pmsg["countenance"], pmsg["signboard"]), None, self.insert_s20, 0, None, ),
-     		( "/%s/[25] %s" % (pmsg["countenance"], pmsg["singing"]), None, self.insert_s25, 0, None, ),
+     		( "/%s/[20] %s" % (self.pmsg["countenance"], self.pmsg["signboard"]), None, self.insert_s20, 0, None, ),
+     		( "/%s/[25] %s" % (self.pmsg["countenance"], self.pmsg["singing"]), None, self.insert_s25, 0, None, ),
      		( "/sep2", None, None, 0, "<Separator>" ),
      
-     		( "/%s" % pmsg["unyu"], None, self.insert_u, 0, None ),
-     		( "/%s" % pmsg["countenance"], None, None, 0, "<Branch>" ),
-     		( "/%s/[10] %s" % (pmsg["countenance"], pmsg["ordinary"]),
+     		( "/%s" % self.pmsg["unyu"], None, self.insert_u, 0, None ),
+     		( "/%s" % self.pmsg["countenance"], None, None, 0, "<Branch>" ),
+     		( "/%s/[10] %s" % (self.pmsg["countenance"], self.pmsg["ordinary"]),
 			  "<control><alt><shift>g", self.insert_s10, 0, None ),
-     		( "/%s/[11] %s" % (pmsg["countenance"], pmsg["opened eyes"]), None, self.insert_s11, 0, None ),
-     		( "/%s/[19] %s" % (pmsg["countenance"], pmsg["singing"]), None, self.insert_s19, 0, None ),
+     		( "/%s/[11] %s" % (self.pmsg["countenance"], self.pmsg["opened eyes"]), None, self.insert_s11, 0, None ),
+     		( "/%s/[19] %s" % (self.pmsg["countenance"], self.pmsg["singing"]), None, self.insert_s19, 0, None ),
      		( "/sep3", None, None, 0, "<Separator>" ),
      
-     		( "/%s" % pmsg["wait"], None, None, 0, "<Branch>" ),
-     		( "/%s/0.1%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w2, 0, None ),
-     		( "/%s/0.2%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w4, 0, None ),
-     		( "/%s/0.3%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w6, 0, None ),
-     		( "/%s/0.4%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w8, 0, None ),
-     		( "/%s/0.5%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w10, 0, None ),
-     		( "/%s/0.6%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w12, 0, None ),
-     		( "/%s/0.9%s" % (pmsg["wait"], pmsg["sec"]), "<control><alt>w", self.insert_w18, 0, None ),
-     		( "/%s/1.0%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w20, 0, None ),
-     		( "/%s/1.2%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w24, 0, None ),
-     		( "/%s/1.5%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w30, 0, None ),
-     		( "/%s/2.0%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w40, 0, None ),
-     		( "/%s/2.5%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w50, 0, None ),
-     		( "/%s/3.0%s" % (pmsg["wait"], pmsg["sec"]), None, self.insert_w60, 0, None ),
+     		( "/%s" % self.pmsg["wait"], None, None, 0, "<Branch>" ),
+     		( "/%s/0.1%s" % (self.pmsg["wait"], self.pmsg["sec"]), None, self.insert_w2, 0, None ),
+     		( "/%s/0.2%s" % (self.pmsg["wait"], self.pmsg["sec"]), None, self.insert_w4, 0, None ),
+     		( "/%s/0.3%s" % (self.pmsg["wait"], self.pmsg["sec"]), None, self.insert_w6, 0, None ),
+     		( "/%s/0.4%s" % (self.pmsg["wait"], self.pmsg["sec"]), None, self.insert_w8, 0, None ),
+     		( "/%s/0.5%s" % (self.pmsg["wait"], self.pmsg["sec"]), None, self.insert_w10, 0, None ),
+     		( "/%s/0.6%s" % (self.pmsg["wait"], self.pmsg["sec"]), None, self.insert_w12, 0, None ),
+     		( "/%s/0.9%s" % (self.pmsg["wait"], self.pmsg["sec"]), "<control><alt>w", self.insert_w18, 0, None ),
+     		( "/%s/1.0%s" % (self.pmsg["wait"], self.pmsg["sec"]), None, self.insert_w20, 0, None ),
+     		( "/%s/1.2%s" % (self.pmsg["wait"], self.pmsg["sec"]), None, self.insert_w24, 0, None ),
+     		( "/%s/1.5%s" % (self.pmsg["wait"], self.pmsg["sec"]), None, self.insert_w30, 0, None ),
+     		( "/%s/2.0%s" % (self.pmsg["wait"], self.pmsg["sec"]), None, self.insert_w40, 0, None ),
+     		( "/%s/2.5%s" % (self.pmsg["wait"], self.pmsg["sec"]), None, self.insert_w50, 0, None ),
+     		( "/%s/3.0%s" % (self.pmsg["wait"], self.pmsg["sec"]), None, self.insert_w60, 0, None ),
      
-     		( "/%s" % pmsg["meta string"], None, None, 0, "<Branch>" ),
-     		( "/%s/%s" % (pmsg["meta string"], pmsg["user name"]), None, self.insert_username, 0, None ),
-     		( "/%s/%s (%s)" % (pmsg["meta string"], pmsg["self name"], pmsg["true"]),
+     		( "/%s" % self.pmsg["meta string"], None, None, 0, "<Branch>" ),
+     		( "/%s/%s" % (self.pmsg["meta string"], self.pmsg["user name"]), None, self.insert_username, 0, None ),
+     		( "/%s/%s (%s)" % (self.pmsg["meta string"], self.pmsg["self name"], self.pmsg["true"]),
 			  None, self.insert_selfname, 0, None ),
-     		( "/%s/%s (%s)" % (pmsg["meta string"], pmsg["self name"], pmsg["false"]),
+     		( "/%s/%s (%s)" % (self.pmsg["meta string"], self.pmsg["self name"], self.pmsg["false"]),
 			  None, self.insert_selfname2, 0, None),
-     		( "/%s/%s" % (pmsg["meta string"], pmsg["kero name"]), None, self.insert_keroname, 0, None ),
+     		( "/%s/%s" % (self.pmsg["meta string"], self.pmsg["kero name"]), None, self.insert_keroname, 0, None ),
      		( "/sep4", None, None, 0, "<Separator>" ),
      
-     		( "/%s" % pmsg["new line"], "<control><alt>n", self.insert_n, 0, None ),
-     		( "/%s" % pmsg["clear"], None, self.insert_c, 0, None ),
+     		( "/%s" % self.pmsg["new line"], "<control><alt>n", self.insert_n, 0, None ),
+     		( "/%s" % self.pmsg["clear"], None, self.insert_c, 0, None ),
      		( "/URL", None, self.insert_url, 0, None ),
      
-			( "/%s" % pmsg["terminate"], "<control><alt>e", self.insert_e, 0, None ),
+			( "/%s" % self.pmsg["terminate"], "<control><alt>e", self.insert_e, 0, None ),
+     		( "/sep5", None, None, 0, "<Separator>" ),
+			( "/%s" % self.msg["edit"][0], None, None, 0, "<Branch>" ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["check"]), None, self.check, 0, "<StockItem>", gtk.STOCK_SPELL_CHECK ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["test"]), None, self.test, 0, "<StockItem>", gtk.STOCK_REFRESH ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["send"]), None, self.send_question, 0, "<StockItem>", gtk.STOCK_OK ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["clear editor"]), None, self.edit_clear, 0, "<StockItem>", gtk.STOCK_CLEAR ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["clear editor all"]), None, self.edit_clear_all, 0, "<StockItem>", gtk.STOCK_CLEAR ),
+     		( "/%s/sep6" % self.msg["edit"][1], None, None, 0, "<Separator>" ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["save phrases"]), None, self.save_phrases, 0, "<StockItem>", gtk.STOCK_SAVE_AS ),
+     		( "/%s/sep7" % self.msg["edit"][1], None, None, 0, "<Separator>" ),
      		)
 		return popup_menu_items
 	
-	def set_self_messages(self):
-		self.msg = {
-			"file":   [unicode(_("File(_F)"), "utf-8"),
-					   unicode(_("File(F)"), "utf-8")],
-			"join":   unicode(_("Join"), "utf-8"),
-			"part":   unicode(_("Part"), "utf-8"),
-			"quit":   unicode(_("Quit"), "utf-8"),
-			"edit":   [unicode(_("Edit(_E)"), "utf-8"),
-					   unicode(_("Edit(E)"), "utf-8")],
-			"cut":	  unicode(_("Cut"), "utf-8"),
-			"copy":   unicode(_("Copy"), "utf-8"),
-			"paste":  unicode(_("Paste"), "utf-8"),
-			"delete": unicode(_("Delete"), "utf-8"),
-			"clear":  unicode(_("Clear"), "utf-8"),
-			"ca":	  unicode(_("Clear All"), "utf-8"),
-			"pref":   [unicode(_("Preferences(_P)"), "utf-8"),
-					   unicode(_("Preferences(P)"), "utf-8")],
-			"channel":[unicode(_("Channel(_C)"), "utf-8"),
-					   unicode(_("Channel(C)"), "utf-8")],
-			"ghost":  [unicode(_("Ghost(_G)"), "utf-8"),
-					   unicode(_("Ghost(G)"), "utf-8")],
-			"window": [unicode(_("Window(_W)"), "utf-8"),
-					   unicode(_("Window(W)"), "utf-8")],
-			"logs":    unicode(_("Logs"), "utf-8"),
-			"phrases": unicode(_("Phrases"), "utf-8"),
-			"gviewer": unicode(_("GViewer"), "utf-8"),
-			"svgmanager": unicode(_("SVGManager"), "utf-8"),
-			"help":   [unicode(_("Help(_H)"), "utf-8"),
-					   unicode(_("Help(H)"), "utf-8")],
-			"about":  unicode(_("About"), "utf-8"),
-		}
-	
 	def get_menu_items(self):
 		menu_items = (
 			( "/%s" % self.msg["file"][0], None, None, 0, "<Branch>" ),
-			( "/%s/%s" % (self.msg["file"][1], self.msg["join"]), "<control>J", self.join, 0, None ),
-			( "/%s/%s" % (self.msg["file"][1], self.msg["part"]), "<control>P", self.part, 0, None ),
+			( "/%s/%s" % (self.msg["file"][1], self.msg["join"]), "<control>J", self.join, 0, "<StockItem>", gtk.STOCK_NETWORK),
+			( "/%s/%s" % (self.msg["file"][1], self.msg["part"]), "<control>P", self.part, 0, "<StockItem>", gtk.STOCK_STOP ),
 			( "/%s/sep1" % self.msg["file"][1], None, None, 0, "<Separator>" ),
-			( "/%s/%s" % (self.msg["file"][1], self.msg["quit"]), "<control>Q", self.quit, 0, None ),
+			( "/%s/%s" % (self.msg["file"][1], self.msg["quit"]), "<control>Q", self.quit, 0, "<StockItem>", gtk.STOCK_QUIT ),
 
 			( "/%s" % self.msg["edit"][0], None, None, 0, "<Branch>" ),
-			( "/%s/%s" % (self.msg["edit"][1], self.msg["cut"]), "<control>X", self.edit_cut, 0, None ),
-			( "/%s/%s" % (self.msg["edit"][1], self.msg["copy"]), "<control>C", self.edit_copy, 0, None ),
-			( "/%s/%s" % (self.msg["edit"][1], self.msg["paste"]), "<control>V", self.edit_paste, 0, None ),
-			( "/%s/%s" % (self.msg["edit"][1], self.msg["delete"]), "<control>D", self.edit_delete,0, None ),
-			( "/%s/sep4" % self.msg["edit"][1], None, None, 0, "<Separator>" ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["check"]), None, self.check, 0, "<StockItem>", gtk.STOCK_SPELL_CHECK ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["test"]), None, self.test, 0, "<StockItem>", gtk.STOCK_REFRESH ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["send"]), None, self.send_question, 0, "<StockItem>", gtk.STOCK_OK ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["clear editor"]), None, self.edit_clear, 0, "<StockItem>", gtk.STOCK_CLEAR ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["clear editor all"]), None, self.edit_clear_all, 0, "<StockItem>", gtk.STOCK_CLEAR ),
+			( "/%s/sep2" % self.msg["edit"][1], None, None, 0, "<Separator>" ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["save phrases"]), None, self.save_phrases, 0, "<StockItem>", gtk.STOCK_SAVE_AS ),
 
-			( "/%s/%s" % (self.msg["edit"][1], self.msg["clear"]), None, self.edit_clear, 0, None ),
-			( "/%s/%s" % (self.msg["edit"][1], self.msg["ca"]), None, self.edit_clear_all, 0, None ),
-			( "/%s/sep5" % self.msg["edit"][1], None, None, 0, "<Separator>" ),
-			( "/%s/%s" % (self.msg["edit"][1], self.msg["pref"][0]), None, self.open_preference, 0, None ),
+			( "/%s/sep3" % self.msg["edit"][1], None, None, 0, "<Separator>" ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["cut"]), "<control>X", self.edit_cut, 0, "<StockItem>", gtk.STOCK_CUT ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["copy"]), "<control>C", self.edit_copy, 0, "<StockItem>", gtk.STOCK_COPY ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["paste"]), "<control>V", self.edit_paste, 0, "<StockItem>", gtk.STOCK_PASTE ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["delete"]), "<control>D", self.edit_delete,0, "<StockItem>", gtk.STOCK_DELETE ),
+			( "/%s/sep4" % self.msg["edit"][1], None, None, 0, "<Separator>" ),
+			( "/%s/%s" % (self.msg["edit"][1], self.msg["pref"][0]), None, self.open_preference, 0, "<StockItem>", gtk.STOCK_PREFERENCES ),
 
 			( "/%s" % self.msg["channel"][0], None, None, 0, "<Branch>" ),
 			( "/%s" % self.msg["ghost"][0], None, None, 0, "<Branch>" ),
@@ -1326,7 +1379,7 @@ class Application:
 			( "/%s/%s" % (self.msg["window"][1], self.msg["svgmanager"]), None, None, 0, None ),
 
 			( "/%s" % self.msg["help"][0], None, None, 0, "<LastBranch>" ),
-			( "/%s/%s" % (self.msg["help"][1], self.msg["about"]), None, self.about, 0, None ),
+			( "/%s/%s" % (self.msg["help"][1], self.msg["about"]), None, self.about, 0, "<StockItem>", gtk.STOCK_DIALOG_INFO ),
 			)
 	
 		return menu_items
