@@ -10,7 +10,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: gvwindow.py,v 1.7 2004/06/23 14:26:52 atzm Exp $
+# $Id: gvwindow.py,v 1.10 2004/07/27 20:04:43 atzm Exp $
 #
 
 import gtk
@@ -158,6 +158,12 @@ class GVWindow:
 		ghost = self.ghostmanager.nominate_ghost(ifghost, self.ghost_dir)
 		self.gv.enqueue(script, ghost, sender, channel, ifghost)
 
+	def get_fmo_ghosts(self):
+		return self.ghostmanager.get_fmo_ghosts()
+
+	def notify_svg_changed(self):
+		self.ghostmanager.generate_dictionary(self.ghost_text)
+
 	def notify_use_gviewer(self, use):
 		if use:
 			self.gv.polling_start()
