@@ -11,7 +11,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: logs.py,v 1.77 2004/06/24 14:22:02 atzm Exp $
+# $Id: logs.py,v 1.83 2004/08/20 06:09:51 atzm Exp $
 #
 
 import os, re, time
@@ -20,13 +20,13 @@ from types import *
 from logfetch     import LogFetcher, LogFetchDialog
 from textmanager  import TextManager
 from xmllogs      import XMLLogger
-from bottleparser import BottleParser
 
 import gtk
 
 from common import *
 
 class LogWindow:
+	TARGET_TIME      = 0
 	TARGET_GHOSTNAME = 1
 	TARGET_CHANNEL   = 2
 	TARGET_SCRIPT    = 5
@@ -104,14 +104,15 @@ class LogWindow:
 
 # === Create Popup Menu for CList === #
 		p_menu = (
-			( "/%s" % unicode(_("Play")), "<control>p", self.play, 0, None ),
+			( "/%s" % unicode(_("Play")), "<control>p", self.play, 0, "<StockItem>", gtk.STOCK_REFRESH ),
 			( "/%s" % unicode(_("Volley")), None, self.volley, 0, None ),
-			( "/%s" % unicode(_("Vote")), "<control><alt>v", self.voting, 0, None ),
-			( "/%s" % unicode(_("Agree")), "<control><alt>a", self.agreeing, 0, None ),
+			( "/%s" % unicode(_("Vote")), "<control><alt>v", lambda x, y: gtk.timeout_add(WAIT_NEXT_REQUEST, self.voting, x, y), 0, "<StockItem>", gtk.STOCK_YES ),
+			( "/%s" % unicode(_("Agree")), "<control><alt>a", lambda x, y: gtk.timeout_add(WAIT_NEXT_REQUEST, self.agreeing, x, y), 0, "<StockItem>", gtk.STOCK_NO ),
 			( '/sep', None, None, 0, "<Separator>" ),
-			( "/%s" % self.msg["copy script"], "<control>c", self.edit_copy_script, 0, None ),
+			( "/%s" % self.msg["copy time"], None, self.edit_copy_time, 0, None ),
 			( "/%s" % self.msg["copy ghostname"], None, self.edit_copy_ghostname, 0, None ),
 			( "/%s" % self.msg["copy channel"], None, self.edit_copy_channel, 0, None ),
+			( "/%s" % self.msg["copy script"], "<control>c", self.edit_copy_script, 0, "<StockItem>", gtk.STOCK_COPY ),
 			( "/%s" % self.msg["copy all"], None, self.edit_copy_all, 0, None ),
 			)
 		self.p_ifact = gtk.ItemFactory(gtk.Menu, "<main>", accel_group)
@@ -123,19 +124,18 @@ class LogWindow:
 		self.ai = self.p_ifact.get_item("/%s" % unicode(_("Agree"), "utf-8"))
 
 # === Create Log Viewer === #
-		main_text = gtk.TextView()
-		main_text.show()
-		main_text.set_cursor_visible(gtk.FALSE)
-		main_text.set_editable(gtk.FALSE)
-		main_text.set_wrap_mode(gtk.WRAP_CHAR)
-		main_text.set_border_width(0)
-		main_text.connect("focus-in-event", self.focus_active_loglist)
+		self.textmanager = TextManager(self, self.app.prefs)
+		self.textmanager.set_cursor_visible(gtk.FALSE)
+		self.textmanager.set_editable(gtk.FALSE)
+		self.textmanager.set_border_width(0)
+		self.textmanager.connect("focus-in-event", self.focus_active_loglist)
+		self.textmanager.show()
 
 		self.sw2 = gtk.ScrolledWindow()
 		self.sw2.show()
 		self.sw2.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
 		self.sw2.set_border_width(0)
-		self.sw2.add(main_text)
+		self.sw2.add(self.textmanager)
 
 		text_frame = gtk.Frame()
 		text_frame.show()
@@ -143,9 +143,6 @@ class LogWindow:
 		text_frame.set_border_width(0)
 		text_frame.add(self.sw2)
 
-		self.textmanager = TextManager(self, main_text, self.app.prefs,
-									   BottleParser('loose'))
-
 # === Create Buttons === #
 		fetch_button = get_icon_button(gtk.STOCK_CONVERT, gtk.ICON_SIZE_MENU)
 		open_button  = get_icon_button(gtk.STOCK_OPEN,    gtk.ICON_SIZE_MENU)
@@ -160,18 +157,15 @@ class LogWindow:
 		save_button.connect("clicked", self.save_as_xml)
 
 		self.pb.connect("clicked", self.play, None)
-		self.vb.connect("clicked", self.voting, None)
-		self.ab.connect("clicked", self.agreeing, None)
+		self.vb.connect("clicked", lambda x, y: gtk.timeout_add(WAIT_NEXT_REQUEST, self.voting, x, y), None)
+		self.ab.connect("clicked", lambda x, y: gtk.timeout_add(WAIT_NEXT_REQUEST, self.agreeing, x, y), None)
 
 		self.vb.set_sensitive(gtk.FALSE)
 		self.ab.set_sensitive(gtk.FALSE)
 
-		fetch_button.connect("focus-in-event", self.focus_active_loglist)
-		open_button.connect("focus-in-event", self.focus_active_loglist)
-		save_button.connect("focus-in-event", self.focus_active_loglist)
-		self.pb.connect("focus-in-event", self.focus_active_loglist)
-		self.vb.connect("focus-in-event", self.focus_active_loglist)
-		self.ab.connect("focus-in-event", self.focus_active_loglist)
+		for item in [fetch_button, open_button, save_button,
+					 self.pb, self.vb, self.ab]:
+			item.connect("focus-in-event", self.focus_active_loglist)
 
 		self.tooltips = gtk.Tooltips()
 		self.tooltips.set_tip(fetch_button, self.msg['fetch'])
@@ -260,16 +254,12 @@ class LogWindow:
 		return gtk.TRUE
 
 	def join(self):
-		self.vb.set_sensitive(gtk.TRUE)
-		self.ab.set_sensitive(gtk.TRUE)
-		self.vi.set_sensitive(gtk.TRUE)
-		self.ai.set_sensitive(gtk.TRUE)
+		for item in [self.vb, self.ab, self.vi, self.ai]:
+			item.set_sensitive(gtk.TRUE)
 
 	def part(self):
-		self.vb.set_sensitive(gtk.FALSE)
-		self.ab.set_sensitive(gtk.FALSE)
-		self.vi.set_sensitive(gtk.FALSE)
-		self.ai.set_sensitive(gtk.FALSE)
+		for item in [self.vb, self.ab, self.vi, self.ai]:
+			item.set_sensitive(gtk.FALSE)
 
 	def color_changed(self, script, sakura, kero, sync, error, URL):
 		self.textmanager.color_changed(script, sakura, kero, sync, error, URL)
@@ -403,6 +393,8 @@ class LogWindow:
 		self.edit_copy(widget, data, self.TARGET_CHANNEL)
 	def edit_copy_script(self, widget, data):
 		self.edit_copy(widget, data, self.TARGET_SCRIPT)
+	def edit_copy_time(self, widget, data):
+		self.edit_copy(widget, data, self.TARGET_TIME)
 	def edit_copy_all(self, widget, data):
 		self.edit_copy(widget, data, self.TARGET_ALL)
 	def edit_copy(self, widget, data, target):
@@ -547,13 +539,13 @@ class LogWindow:
 		mid, type = self.logbook.voting(widget, data, self.window)
 		if mid is None or type is None:
 			return
-		gtk.timeout_add(0, self.app.vote_message, mid, type)
+		gtk.timeout_add(WAIT_NEXT_REQUEST, self.app.vote_message, mid, type)
 
 	def agreeing(self, widget, data):
 		mid, type = self.logbook.agreeing(widget, data, self.window)
 		if mid is None or type is None:
 			return
-		gtk.timeout_add(0, self.app.vote_message, mid, type)
+		gtk.timeout_add(WAIT_NEXT_REQUEST, self.app.vote_message, mid, type)
 
 	def button_popup(self, widget, event, sender):
 		if event.button in [1, 3]:
@@ -640,26 +632,32 @@ class LogWindow:
 	def get_menu_items(self):
 		menu_items = (
 			( "/%s" % self.app.msg["file"][0], None, None, 0, "<Branch>" ),
-			( "/%s/%s" % (self.app.msg["file"][1], self.msg["open_xml"]), "<control>o", self.open_xml_log, 0, None ),
-			( "/%s/%s" % (self.app.msg["file"][1], self.msg["save"]), "<control>s", self.save_as_xml, 0, None ),
-			( "/%s/%s" % (self.app.msg["file"][1], self.msg["fetch"]), None, self.fetch, 0, None ),
-			( "/%s/%s" % (self.app.msg["file"][1], self.msg["close"]), "<control>q", self.close, 0, None ),
+			( "/%s/%s" % (self.app.msg["file"][1], self.msg["open_xml"]), "<control>o", self.open_xml_log,
+			  0, "<StockItem>", gtk.STOCK_OPEN ),
+			( "/%s/%s" % (self.app.msg["file"][1], self.msg["save"]), "<control>s", self.save_as_xml,
+			  0, "<StockItem>", gtk.STOCK_SAVE_AS ),
+			( "/%s/%s" % (self.app.msg["file"][1], self.msg["fetch"]), None, self.fetch,
+			  0, "<StockItem>", gtk.STOCK_CONVERT ),
+			( "/%s/%s" % (self.app.msg["file"][1], self.msg["close"]), "<control>q", self.close,
+			  0, "<StockItem>", gtk.STOCK_CLOSE ),
 
 			( "/%s" % self.app.msg["edit"][0], None, None, 0, "<Branch>" ),
-			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["copy script"]),
-			  "<control>c", self.edit_copy_script, 0, None ),
+			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["copy time"]),
+			  None, self.edit_copy_time, 0, None ),
 			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["copy ghostname"]),
 			  None, self.edit_copy_ghostname, 0, None ),
 			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["copy channel"]),
 			  None, self.edit_copy_channel, 0, None ),
+			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["copy script"]),
+			  "<control>c", self.edit_copy_script, 0, "<StockItem>", gtk.STOCK_COPY ),
 			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["copy all"]),
 			  None, self.edit_copy_all, 0, None ),
 			( "/%s/sep1" % self.app.msg["edit"][1], None, None, 0, "<Separator>" ),
 			( "/%s/%s" % (self.app.msg["edit"][1], self.msg["search"]), "<control>f",
-			  lambda x, y: self.search_entry.grab_focus(), 0, None ),
+			  lambda x, y: self.search_entry.grab_focus(), 0, "<StockItem>", gtk.STOCK_FIND ),
 			( "/%s/sep2" % self.app.msg["edit"][1], None, None, 0, "<Separator>" ),
 			( "/%s/%s" % (self.app.msg["edit"][1], self.app.msg["pref"][0]), None,
-			  self.app.open_preference, 0, None ),
+			  self.app.open_preference, 0, "<StockItem>", gtk.STOCK_PREFERENCES ),
 
 			( "/%s" % self.msg["view"][0], None, None, 0, "<Branch>" ),
 
@@ -679,7 +677,8 @@ class LogWindow:
 							 self.msg["script style"])),
 
 			( "/%s" % self.app.msg["help"][0], None, None, 0, "<LastBranch>" ),
-			( "/%s/%s" % (self.app.msg["help"][1], self.app.msg["about"]), None, self.app.about, 0, None ),
+			( "/%s/%s" % (self.app.msg["help"][1], self.app.msg["about"]), None, self.app.about,
+			  0, "<StockItem>", gtk.STOCK_DIALOG_INFO ),
 			)
 		return menu_items
 
@@ -692,6 +691,7 @@ class LogWindow:
 			"copy script":   unicode(_("Copy script"), "utf-8"),
 			"copy ghostname": unicode(_("Copy ghost name"), "utf-8"),
 			"copy channel":   unicode(_("Copy channel"), "utf-8"),
+			"copy time":   unicode(_("Copy time"), "utf-8"),
 			"copy all":   unicode(_("Copy all"), "utf-8"),
 			"search":   unicode(_("Search"), "utf-8"),
 
@@ -886,7 +886,7 @@ class LogList(gtk.CList):
 
 		self.set_shadow_type(gtk.SHADOW_IN)
 		self.set_selection_mode(gtk.SELECTION_BROWSE)
-		self.set_reorderable(gtk.TRUE)
+		self.set_reorderable(gtk.FALSE)
 		self.set_size_request(450, 120)
 		self.set_column_width(0, 100)
 		self.column_titles_passive()
