@@ -10,7 +10,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: textmanager.py,v 1.30 2004/05/24 23:48:40 atzm Exp $
+# $Id: textmanager.py,v 1.32 2004/08/20 06:09:51 atzm Exp $
 #
 
 import os
@@ -21,16 +21,20 @@ from bottleparser import *
 from types import *
 from common import *
 
-class TextManager:
+class TextManager(gtk.TextView):
 	FONT_SCRIPT = 'Monospace'
 
-	def __init__(self, app, textview, prefs, parser=BottleParser('loose')):
+	def __init__(self, app, prefs, parser=BottleParser('loose')):
+		gtk.TextView.__init__(self)
+
 		self.app = app
-		self.textview = textview
 		self.parser = parser
 		self.view_style = STYLE_TALK
 		self.prefs = prefs
-		self.textbuffer = self.textview.get_buffer()
+		self.textbuffer = gtk.TextView.get_buffer(self)
+
+		# settings
+		self.set_wrap_mode(gtk.WRAP_CHAR)
 
 		# create tags
 		self.textbuffer.create_tag('script')
@@ -47,7 +51,7 @@ class TextManager:
 		# some connection
 		self.p_flag = False
 		url_tag.connect('event', self.receive_event)
-		self.textview.connect('motion-notify-event', self.motion)
+		self.connect('motion-notify-event', self.motion)
 		self.app.window.connect('focus-out-event', self.focus_out)
 
 		# initialize
@@ -104,6 +108,10 @@ class TextManager:
 		else:
 			self.p_flag = False
 
+	def get_script(self):
+		[s, e] = self.textbuffer.get_bounds()
+		return self.textbuffer.get_text(s, e)
+
 	RE_NEWLINE = re.compile('\n+$')
 	def coloring(self, widget=None, event=None):
 		current = ['sakura'] # tag stack for closable (currently only synchronized) sessions
@@ -187,9 +195,9 @@ class TextManager:
 		#self.textbuffer.insert(self.textbuffer.get_end_iter(), '\n')
 		if end != pos:
 			if end != 0:
-				self.textview.emit('move-cursor', gtk.MOVEMENT_VISUAL_POSITIONS, end*-1, gtk.FALSE)
+				self.emit('move-cursor', gtk.MOVEMENT_VISUAL_POSITIONS, end*-1, gtk.FALSE)
 			if pos != 0:
-				self.textview.emit('move-cursor', gtk.MOVEMENT_VISUAL_POSITIONS, pos, gtk.FALSE)
+				self.emit('move-cursor', gtk.MOVEMENT_VISUAL_POSITIONS, pos, gtk.FALSE)
 
 	def insert_with_color_from(self, string, pos=-3):
 		current = ['sakura'] # tag stack for closable (currently only synchronized) sessions
@@ -289,7 +297,7 @@ class TextManager:
 		self.textbuffer.apply_tag_by_name('font', si, ei)
 
 		if pos != 0:
-			self.textview.emit("move-cursor", gtk.MOVEMENT_VISUAL_POSITIONS, pos, gtk.FALSE)
+			self.emit("move-cursor", gtk.MOVEMENT_VISUAL_POSITIONS, pos, gtk.FALSE)
 
 	def set_style_talk(self):
 		self.view_style = STYLE_TALK
@@ -316,3 +324,6 @@ class TextManager:
 
 	def clear(self):
 		self.textbuffer.set_text("")
+
+	def get_buffer(self):
+		return self.textbuffer
