@@ -12,7 +12,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: ghost.py,v 1.7 2004/07/21 05:00:12 atzm Exp $
+# $Id: ghost.py,v 1.8 2004/08/27 18:14:03 tkawa Exp $
 #
 
 import gtk
@@ -209,7 +209,7 @@ class Ghost:
 		if self.debug:
 			print "%-14s: %s" % (desc, text)
 
-	EXPAND_QUOTE_REGEX = re.compile('"(.+)"')
+	EXPAND_QUOTE_REGEX = re.compile('"(.*)"')
 	def expand_quote(self, text):
 		m = self.EXPAND_QUOTE_REGEX.match(text)
 		if m is not None:
