@@ -11,7 +11,7 @@
 #  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
 #  PURPOSE.  See the GNU General Public License for more details.
 #
-# $Id: ghostmanager.py,v 1.2 2004/06/20 23:46:55 atzm Exp $
+# $Id: ghostmanager.py,v 1.8 2004/07/27 19:55:45 atzm Exp $
 #
 
 import os
@@ -25,6 +25,8 @@ from viewercommon import *
 class GhostManager:
 	def __init__(self, textfile='ghost.txt', debug=DEBUG):
 		self.ghost_dic = {}
+		self.fmo_ghost_list = []
+
 		self.generate_dictionary(textfile)
 		self.debug = debug
 
@@ -46,12 +48,27 @@ class GhostManager:
 				print 'no such ghost: %s' % ifghost
 			return None
 
+	def nominate_ghost_txt(self, svg_txt, svg_dir='Ghost'):
+		try:
+			return Ghost(svg_txt, svg_dir)
+		except KeyError:
+			if self.debug:
+				print '-' * 20
+				print 'no such svg_txt: %s' % svg_txt
+			return None
+
+	def get_fmo_ghosts(self):
+		return self.fmo_ghost_list
+
 	def generate_dictionary(self, textfile='ghost.txt'):
 		try:
 			file = open(textfile)
 		except IOError:
 			return -1
 
+		self.ghost_dic = {}
+		self.fmo_ghost_list = []
+
 		base_dir = os.path.dirname(textfile)
 		quote = re.compile('"(.+?)"')
 		while 1:
@@ -79,3 +96,8 @@ class GhostManager:
 			path = os.path.normpath(path)
 			ghostname = unicode(ghostname, 'sjis', 'replace')
 			self.ghost_dic[ghostname] = path
+
+			if fmo == "FMO":
+				if ghostname in [NAME_SAKURA_KAWAKAMI, NAME_SAKURA_KUROI]:
+					ghostname = NAME_SAKURA
+				self.fmo_ghost_list.append(ghostname)
